@@ -184,10 +184,22 @@ def addsitepackages(known_paths, sys_prefix=sys.prefix, exec_prefix=sys.exec_pre
     prefixes = [os.path.join(sys_prefix, "local"), sys_prefix]
     if exec_prefix != sys_prefix:
         prefixes.append(os.path.join(exec_prefix, "local"))
+
     for prefix in prefixes:
         if prefix:
             if sys.platform in ('os2emx', 'riscos') or sys.platform[:4] == 'java':
                 sitedirs = [os.path.join(prefix, "Lib", "site-packages")]
+            elif sys.platform == 'darwin' and prefix == sys_prefix:
+
+                if prefix.startswith("/System/Library/Frameworks/"): # Apple's Python
+
+                    sitedirs = [os.path.join("/Library/Python", sys.version[:3], "site-packages"),
+                                os.path.join(prefix, "Extras", "lib", "python")]
+
+                else: # any other Python distros on OSX work this way
+                    sitedirs = [os.path.join(prefix, "lib",
+                                             "python" + sys.version[:3], "site-packages")]
+
             elif os.sep == '/':
                 sitedirs = [os.path.join(prefix,
                                          "lib",
@@ -408,9 +420,20 @@ def virtual_install_main_packages():
     # This is hardcoded in the Python executable, but
     # relative to sys.prefix, so we have to fix up:
     for path in list(paths):
-        tk_dir = os.path.join(paths[0], 'lib-tk')
+        tk_dir = os.path.join(path, 'lib-tk')
         if os.path.exists(tk_dir):
             paths.append(tk_dir)
+
+    # These are hardcoded in the Apple's Python executable,
+    # but relative to sys.prefix, so we have to fix them up:
+    if sys.platform == 'darwin':
+        hardcoded_paths = [os.path.join(sys.real_prefix, 'lib', 'python'+sys.version[:3], module)
+                           for module in ('plat-darwin', 'plat-mac', 'plat-mac/lib-scriptpackages')]
+
+        for path in hardcoded_paths:
+            if os.path.exists(path):
+                paths.append(path)
+
     sys.path.extend(paths)
 
 def virtual_addsitepackages(known_paths):
@@ -419,6 +442,15 @@ def virtual_addsitepackages(known_paths):
     else:
         return known_paths
 
+def fixclasspath():
+    """Adjust the special classpath sys.path entry for Jython. The classpath
+    must follow the virtualenv libs
+    """
+    classpath = '__classpath__'
+    if classpath in sys.path:
+        sys.path.remove(classpath)
+        sys.path.append(classpath)
+
 def main():
     virtual_install_main_packages()
     abs__file__()
@@ -428,6 +460,8 @@ def main():
         addbuilddir()
     paths_in_sys = addsitepackages(paths_in_sys)
     paths_in_sys = virtual_addsitepackages(paths_in_sys)
+    if sys.platform[:4] == 'java':
+        fixclasspath()
     if sys.platform == 'os2emx':
         setBEGINLIBPATH()
     setquit()
